# s6-rc service management
Puppet::Type.type(:service).provide :s6rc, :parent => :base do
  desc <<-'EOT'
    s6-rc service management.

    This provider manages daemons supervised by Laurent Bercot s6 supervision suite.

    This provider supports:

    * start/stop/restart
    * enable/disable
    * mask/unmask
    * status active/enabled/masked
  EOT

  commands :svctl => "svctl"

  def self.instances
    i = []
    output = svctl(["listall"])
    output.scan(/^(\S+)$/i).each do |m|
      i << new(:name => m[0])
    end
    return i
  rescue Puppet::ExecutionFailure
    return []
  end

  def status
    begin
      svctl(["isup", @resource[:name]])
      Puppet.debug("#{@resource[:name]} is running")
      return :running
    rescue Puppet::ExecutionFailure => e
      if $?.exitstatus == 1
        Puppet.debug("#{@resource[:name]} is stopped")
        return :stopped
      else
        Puppet.err("#{@resource[:name]} #{__method__}: #{e.inspect}")
        return nil
      end
    end
  end

  def start
    begin
      svctl(["start", @resource[:name]])
      Puppet.debug("#{@resource[:name]} has been started")
    rescue Puppet::ExecutionFailure => e
      Puppet.err("#{@resource[:name]} #{__method__}: #{e.inspect}")
      return nil
    end
  end

  def stop
    begin
      svctl(["stop", @resource[:name]])
      Puppet.debug("#{@resource[:name]} has been stopped")
    rescue Puppet::ExecutionFailure => e
      Puppet.err("#{@resource[:name]} #{__method__}: #{e.inspect}")
      return nil
    end
  end

  def restart
    begin
      svctl(["restart", @resource[:name]])
      Puppet.debug("#{@resource[:name]} has been restarted")
    rescue Puppet::ExecutionFailure => e
      Puppet.err("#{@resource[:name]} #{__method__}: #{e.inspect}")
      return nil
    end
  end

  def enabled?
    begin
      svctl(["isenabled", @resource[:name]])
      Puppet.debug("#{@resource[:name]} is enabled")
      return :true
    rescue Puppet::ExecutionFailure => e
      if $?.exitstatus == 1
        Puppet.debug("#{@resource[:name]} is disabled")
        return :false
      else
        Puppet.err("#{@resource[:name]} #{__method__}: #{e.inspect}")
        return nil
      end
    end
  end

  def enable
    begin
      svctl(["enable", @resource[:name]])
      self.updatecmd
      Puppet.debug("#{@resource[:name]} has been enabled")
    rescue Puppet::ExecutionFailure => e
      Puppet.err("#{@resource[:name]} #{__method__}: #{e.inspect}")
      return nil
    end
  end

  def disable
    begin
      svctl(["disable", @resource[:name]])
      self.updatecmd
      Puppet.debug("#{@resource[:name]} has been disabled")
    rescue Puppet::ExecutionFailure => e
      Puppet.err("#{@resource[:name]} #{__method__}: #{e.inspect}")
      return nil
    end
  end

  def masked?
    begin
      svctl(["ismasked", @resource[:name]])
      Puppet.debug("#{@resource[:name]} is masked")
      return :true
    rescue Puppet::ExecutionFailure => e
      if $?.exitstatus == 1
        Puppet.debug("#{@resource[:name]} is not masked")
        return :false
      else
        Puppet.err("#{@resource[:name]} #{__method__}: #{e.inspect}")
        return nil
      end
    end
  end

  def mask
    begin
      svctl(["mask", @resource[:name]])
      self.updatecmd
      Puppet.debug("#{@resource[:name]} has been masked")
    rescue Puppet::ExecutionFailure => e
      Puppet.err("#{@resource[:name]} #{__method__}: #{e.inspect}")
      return nil
    end
  end

  def unmask
    begin
      svctl(["unmask", @resource[:name]])
      self.updatecmd
      Puppet.debug("#{@resource[:name]} has been unmasked")
    rescue Puppet::ExecutionFailure => e
      Puppet.err("#{@resource[:name]} #{__method__}: #{e.inspect}")
      return nil
    end
  end

  def updatecmd
    begin
      svctl(["update"])
    rescue Puppet::ExecutionFailure => e
      Puppet.err("#{@resource[:name]} #{__method__}: #{e.inspect}")
      return nil
    end
  end
end
